import { useLocation, useNavigate } from "react-router-dom"
import { useGlobalStore } from '@/store/global';
import { lazy, useEffect, useState } from 'react';
import GloablLoading from '@/components/global-loading';
import Slide from './slide';
import Header from './header';
import { useUserStore } from '@/store/global/user';
import { useMenusStore } from "@/store/global/menu";
import { Menu } from '@/models';
import { components, commonRoutes } from '@/router/config';

import { replaceRoutes, router } from '@/router/router';
import Result404 from './404';

import './index.css'
import TabsLayout from './tabs-layout';
import Content from './content';
import userService from '@/request/service/user';
import authService from '@/request/service/auth';
import { useRequest } from "@/hooks/use-request";



const BasicLayout: React.FC = () => {

  const [loading, setLoading] = useState(true);

  const { refreshToken, lang, token } = useGlobalStore();
  const { currentUser, setCurrentUser } = useUserStore();
  const { setMenus } = useMenusStore();
  const navigate = useNavigate();
  const location = useLocation();

  const {
    data: currentUserProfile,
    run: getCurrentUserProfile,
  } = useRequest(
    userService.getProfile,
    { manual: true }
  );

  const { data: menuList, run: listMenus} = useRequest(authService.listMenus, { manual: true });

  useEffect(() => {
    if (!refreshToken) {
      navigate('/login');
      return;
    }
    getCurrentUserProfile();
    listMenus();
  }, [refreshToken, getCurrentUserProfile, listMenus, navigate]);

  useEffect(() => {
    if (!currentUserProfile || !menuList?.data) return;
    const routes: Menu[] = [];

    const fixAndPushMenu = (item: Menu) => {
      if(!item.parentPaths) {
        item.parentPaths = []
      }

      if(!item.path.startsWith('/')) {
        item.path = `/${item.path}`
      }
      const parentPath = item.parentPaths.join('')
      if(!item.path.startsWith(parentPath)) {
        item.path = [...item.parentPaths, item.path].join('')
      }
      
      if(item.component) {
        item.component = `${item.component.startsWith('/')?'':'/'}${item.component}${item.component.endsWith('.tsx')?'':'.tsx'}`
      }
      routes.push(item)
      if(item.children && item.children.length > 0) {
        const parentPaths= item.path.replace('/index.tsx', '').split("/").filter(it => it!== '').map(it => `/${it}`)
        item.children.forEach(it => {
          it.parentPaths = parentPaths
          fixAndPushMenu(it)
        })
      }
    }
    menuList.data.forEach(item => fixAndPushMenu(item));
    setMenus(menuList.data);
    replaceRoutes('*', [
      ...[...routes, ...(commonRoutes as Menu[])].map(menu => ({
        path: `/*${menu.path}`,
        Component: menu.component && menu.component in components? lazy(components[menu.component]) : Result404,
        id: `/*${menu.path}`,
        handle: {
          parentPaths: menu.parentPaths,
          path: menu.path,
          name: menu.name,
          icon: menu.icon,
        },
      })), {
        id: '*',
        path: '*',
        Component: Result404,
        handle: {
          path: '404',
          name: '404',
        },
      },
    ]);
    setCurrentUser(currentUserProfile!!.data);
    setLoading(false);

    // replace一下当前路由，为了触发路由匹配
    router.navigate(`${location.pathname}${location.search}`, { replace: true });
  }, [ currentUserProfile, menuList, setCurrentUser ]);

  useEffect(() => {
    function storageChange(e: StorageEvent) {
      if (e.key === useGlobalStore.persist.getOptions().name) {
        useGlobalStore.persist.rehydrate();
      }
    }

    window.addEventListener<'storage'>('storage', storageChange);

    return () => {
      window.removeEventListener<'storage'>('storage', storageChange);
    }
  }, []);

  return (loading || !currentUser)? (<GloablLoading />) : (
    <div>
      <div key={lang} className='bg-primary overflow-hidden'>
        <Header />
        <Slide />
        <Content>
          <TabsLayout />
        </Content>
      </div>
    </div>
  );
};

export default BasicLayout;
