import React, { useState, useEffect, useRef } from 'react';
import { useSetState } from 'ahooks';
import { Space, Table, Button, Divider, Card, Input } from 'antd';
import type { InputRef } from 'antd';
import type { ColumnsType, ColumnType, TableProps } from 'antd/es/table';
import { t } from '@/utils/i18n';
import { PlusOutlined, ExclamationCircleFilled, SearchOutlined } from '@ant-design/icons';
import { antdUtils } from '@/utils/antd';
import { useRequest } from '@/hooks/use-request';
import { TemplateDictVO, TemplateDictPageReqVO } from '@/models';
import DictEditor from './dict-editor';
import templateDictService from '@/request/service/template-dict';

export default (props: {
    onSelectItem: (item: TemplateDictVO) => void;
}) => {
    const { onSelectItem } = props;
    const [dataDicts, setDataDicts] = useState<TemplateDictVO[]>();
    const [searchState, setSearchState] = useSetState<TemplateDictPageReqVO>({
        pageNo: 1,
        pageSize: 10
    });
    const [total, setTotal] = useState(0)
    const searchInput = useRef<InputRef>(null);
    const [onSearching, setOnSearching] = useState(false);

    const [editorVisable, seEditorVisable] = useState<boolean>(false);
    const [editData, seEditData] = useState<TemplateDictVO>();

    //获取字典分页数据
    const { runAsync: getDictPageApi } = useRequest(templateDictService.getDictPageApi, { manual: true });
    //删除字典数据
    const { runAsync: deleteDictApi } = useRequest(templateDictService.deleteDictApi, { manual: true });

    const load = async () => {
        setOnSearching(true);
        const [error, { data }] = await getDictPageApi(searchState);
        setOnSearching(false);
        if (!error) {
            setDataDicts(data.list);
            setTotal(data.total);
        }
    }

    const showDeleteConfirm = (item: TemplateDictVO) => {
        antdUtils.modal?.confirm({
            title: `确认删除名称为: ${item.name} 的字典吗?`,
            icon: <ExclamationCircleFilled />,
            content: '请注意删除以后不可恢复！',
            okText: '删除',
            okType: 'danger',
            cancelText: '取消',
            onOk: async () => {
                const [error, { code, msg }] = await deleteDictApi(item.id);
                if (error || code !== 0) {
                    antdUtils.message?.open({ type: 'error', content: msg ?? '操作失败' })
                } else {
                    antdUtils.message?.open({ type: 'success', content: '删除成功' })
                }
                await load();
            },
            onCancel() {
            },
        });
    };

    const getColumnSearchProps = (placeholder: string): ColumnType<TemplateDictVO> => ({
        filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => (
            <div style={{ padding: 8 }} onKeyDown={(e) => e.stopPropagation()}>
                <Input.Search
                    ref={searchInput}
                    placeholder={placeholder}
                    value={selectedKeys[0]}
                    onChange={(e) => {
                        setSelectedKeys(e.target.value && e.target.value !== '' ? [e.target.value] : [])
                    }}
                    onSearch={(value) => {
                        if (value === '' && clearFilters) {
                            clearFilters!!()
                        }
                        confirm();
                    }}
                    onPressEnter={() => confirm()}
                    allowClear
                    style={{ marginBottom: 8, display: 'block' }}
                    enterButton="搜索"
                    size="middle"
                    loading={onSearching}
                />
            </div>
        ),
        filterIcon: (filtered: boolean) => (
            <SearchOutlined style={{ color: filtered ? 'primaryColor' : undefined }} />
        ),
        onFilterDropdownOpenChange: (visible) => {
            if (visible) {
                setTimeout(() => searchInput.current?.select(), 100);
            }
        },
    });

    const columns: ColumnsType<TemplateDictVO> = [
        {
            title: '名称',
            dataIndex: 'name',
            key: 'name',
            align: 'center',
            ...getColumnSearchProps('请输入名称'),
        },
        {
            title: '描述',
            key: 'description',
            dataIndex: 'description',
            align: 'center',
            // ...getColumnSearchProps('请输入描述'),
        },
        {
            title: t("QkOmYwne" /* 操作 */),
            key: 'action',
            align: 'center',
            render: (_, record) => (
                <Space size="middle" split={(
                    <Divider type='vertical' />
                )}>
                    <a
                        onClick={() => {
                            seEditData(record)
                            seEditorVisable(true);
                        }}>
                        编辑
                    </a>
                    <a
                        onClick={() => {
                            showDeleteConfirm(record)
                        }}>
                        删除
                    </a>
                </Space>
            ),
            width: 150,
        },
    ];

    useEffect(() => {
        load();
    }, [searchState]);

    const onChange: TableProps<TemplateDictVO>['onChange'] = (pagination, filters, sorter, extra) => {
        const state: TemplateDictPageReqVO = {
            name: filters.name ? filters.name[0] as string : undefined,
            description: filters.description ? filters.description[0] as string : undefined,
            pageNo: pagination.current,
            pageSize: pagination.pageSize
        }
        setOnSearching(true);
        setSearchState(state);
    };

    return (
        <>
            <div>
                <Card className='mb-[10px] dark:bg-[rgb(33,41,70)] bg-white roundle-lg px[12px]' bodyStyle={{
                    paddingTop: 0,
                    paddingBottom: 0
                }}>
                    <div className="py-[8px] flex flex-row-reverse">
                        <Button className="ml-5" type='primary' size='middle' icon={<PlusOutlined />}
                            onClick={() => {
                                seEditData(undefined);
                                seEditorVisable(true);
                            }}> 新增 </Button>
                    </div>
                    <Table rowKey="id" scroll={{ x: true }} columns={columns} dataSource={dataDicts} className='bg-transparent'
                        onRow={(record) => {
                            return {
                                onClick: () => {
                                    onSelectItem(record);
                                }
                            };
                        }}
                        onChange={onChange}
                        pagination={{ 
                            current: searchState.pageNo,
                            pageSize: searchState.pageSize,
                            total,
                            position: ['bottomRight'] 
                        }}
                    />
                </Card>
            </div>
            <DictEditor
                onSave={() => {
                    load();
                    seEditorVisable(false);
                }}
                onCancel={() => { seEditorVisable(false) }}
                visible={editorVisable}
                data={editData}
            />
        </>
    );
};
