import React, { useEffect, useState } from 'react'
import { Form, Input, InputNumber, Modal } from 'antd';
import dictService from '@/request/service/template-dict-detail';
import { useRequest } from '@/hooks/use-request';
import type { TemplateDictDetailVO } from '@/models'
import { antdUtils } from '@/utils/antd';

const layout = {
    labelCol: { span: 4, },
    wrapperCol: { span: 16 }
};

export default (props: {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (role: TemplateDictDetailVO) => void;
    dictId?: number;
    data?: TemplateDictDetailVO | null;
}) => {

    const { visible, onCancel, onSave, dictId, data } = props;

    const { runAsync: updateApi } = useRequest(dictService.updateApi, { manual: true });
    const { runAsync: createApi } = useRequest(dictService.createApi, { manual: true });

    const isEdit = !!data;


    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();

    useEffect(() => {
        if (visible) {
            if (data) {
                form.setFieldsValue(data);
            }
        } else {
            form.resetFields();
        }
    }, [visible]);

    const save = async () => {
        setSaveLoading(true);
        const fieldValues = form.getFieldsValue();
        const newValue = isEdit ? { ...data, ...fieldValues } : {...fieldValues, dictId};
        const [error, { msg, code }] = isEdit ? await updateApi(newValue) : await createApi(newValue);
        if (!error && code === 0) {
            onSave(newValue);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg ?? '操作失败',
            });
        }
        setSaveLoading(false);
    }

    return (
        <>
            <Modal
                open={visible}
                title={isEdit ? "编辑" : "新建"}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                confirmLoading= {saveLoading}
                destroyOnClose
            >
                <Form
                    form={form}
                    {...layout}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                >
                    <Form.Item name="label" label="字典标签"
                        rules={[
                            {
                                required: true,
                                message: '请输字典标签',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="value" label="字典值"
                        rules={[
                            {
                                required: true,
                                message: '请输入字典值',
                            },
                        ]}
                    >
                        <Input.TextArea />
                    </Form.Item>

                    <Form.Item name="dictSort" label="字典排序"
                        rules={[
                            {
                                required: true,
                                message: '请输入字典排序',
                            },
                        ]}
                    >
                        <InputNumber min={1} />
                    </Form.Item>
                </Form>
            </Modal>
        </>
    )
};
