import React, { useState, useEffect, useRef, useContext, MutableRefObject, forwardRef, useImperativeHandle } from 'react';
import { Space, Table, Button, Input, Badge, Divider, Form, Popconfirm, Modal, Radio } from 'antd';
import type { TableColumnsType } from 'antd';
import type { InputRef } from 'antd';
import type { FormInstance } from 'antd/es/form';
import type { ColumnType, TableProps } from 'antd/es/table';
import { t } from '@/utils/i18n';
import { PlusOutlined, ExclamationCircleFilled, SearchOutlined, UndoOutlined } from '@ant-design/icons';
import type { GoodsClassifyPageReqVO, GoodsClassifyVO } from '@/models'
import { antdUtils } from '@/utils/antd';
import { useRequest } from '@/hooks/use-request';
import goodsClassifyService from '@/request/service/goods-classify';
import { formatDate } from '@/utils/formatTime';
import { useSetState } from 'ahooks';


const CreateClassify = (props: {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (role: GoodsClassifyVO) => void;
}) => {

    const { visible, onCancel, onSave } = props;

    const { runAsync: createApi } = useRequest(goodsClassifyService.createGoodsClassifyApi, { manual: true });


    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();

    useEffect(() => {
        if (visible) {
            form.setFieldValue("isDefault", 1);
        } else {
            form.resetFields();
        }
    }, [visible]);

    const save = async () => {
        setSaveLoading(true);
        const fieldValues = form.getFieldsValue();
        const [error, { msg, code }] = await createApi(fieldValues);
        if (!error && code === 0) {
            onSave(fieldValues);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg ?? '操作失败',
            });
        }
        setSaveLoading(false);
    }

    return (
        <>
            <Modal
                open={visible}
                title={"新建"}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                confirmLoading={saveLoading}
                destroyOnClose
            >
                <Form
                    form={form}
                    // layout={{
                    //     labelCol: { span: 4, },
                    //     wrapperCol: { span: 16 }
                    // }}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                >
                    <Form.Item name="classifyName" label="分类名称:"
                        rules={[
                            {
                                required: true,
                                message: '请输分类名称',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>
                    <Form.Item name="isDefault" label="是否默认:"
                        rules={[
                            {
                                required: true,
                                message: '请输选择',
                            },
                        ]}
                    >
                        <Radio.Group options={[
                            { value: 1, label: "是" },
                            { value: 2, label: "否" }
                        ]} optionType="default">
                        </Radio.Group>
                    </Form.Item>
                </Form>
            </Modal>
        </>
    )
};

const EditableContext = React.createContext<FormInstance<any> | null>(null);

interface EditableRowProps {
    index: number;
}

const EditableRow: React.FC<EditableRowProps> = ({ index, ...props }) => {
    const [form] = Form.useForm();
    return (
        <Form form={form} component={false}>
            <EditableContext.Provider value={form}>
                <tr {...props} />
            </EditableContext.Provider>
        </Form>
    );
};

interface EditableCellProps {
    title: React.ReactNode;
    editable: boolean;
    children: React.ReactNode;
    dataIndex: keyof GoodsClassifyVO;
    record: GoodsClassifyVO;
    handleSave: (record: GoodsClassifyVO) => void;
    onEditing: (record: GoodsClassifyVO) => void;
}

const EditableCell: React.FC<EditableCellProps> = ({
    title,
    editable,
    children,
    dataIndex,
    record,
    handleSave,
    ...restProps
}) => {
    const [editing, setEditing] = useState(false);
    const inputRef = useRef<InputRef>(null);
    const form = useContext(EditableContext)!;

    useEffect(() => {
        if (editing) {
            inputRef.current!.focus();
        }
    }, [editing]);

    const toggleEdit = () => {
        setEditing(!editing);
        form.setFieldsValue({ [dataIndex]: record[dataIndex] });
    };

    const save = async () => {
        try {
            const values = await form.validateFields();

            toggleEdit();
            handleSave({ ...record, ...values });
        } catch (errInfo) {
            console.log('Save failed:', errInfo);
        }
    };

    let childNode = children;

    if (editable) {
        childNode = editing ? (
            <Form.Item
                style={{ margin: 0 }}
                name={dataIndex}
                rules={[
                    {
                        required: true,
                        message: `${title} is required.`,
                    },
                ]}
            >
                <Input size="middle" ref={inputRef} onPressEnter={save} onBlur={save} />
            </Form.Item>
        ) : (
            <div className="editable-cell-value-wrap" style={{ paddingRight: 24 }} onClick={toggleEdit}>
                {children}
            </div>
        );
    }

    return <td {...restProps}>{childNode}</td>;
};

type EditableTableProps = Parameters<typeof Table>[0];
type ColumnTypes = Exclude<EditableTableProps['columns'], undefined>;

export default forwardRef((props, ref) => {
    //TODO: pagination
    const [visible, setVisible] = useState<boolean>(false);
    const [dataSource, setDataSource] = useState<GoodsClassifyVO[]>([]);
    const [searchFrom] = Form.useForm();

    const [searchState, setSearchState] = useSetState<GoodsClassifyPageReqVO>({
        pageNo: 1,
        pageSize: 10
    });
    const [total, setTotal] = useState(0)
    const searchInput = useRef<InputRef>(null);
    const [onSearching, setOnSearching] = useState(false);

    const { runAsync: getPageApi } = useRequest(goodsClassifyService.getGoodsClassifyPageApi, { manual: true });
    const { runAsync: updateApi } = useRequest(goodsClassifyService.updateGoodsClassifyApi, { manual: true });
    const { runAsync: verifyApi } = useRequest(goodsClassifyService.classifyNameVerifyUnique, { manual: true });
    const { runAsync: deleteApi } = useRequest(goodsClassifyService.deleteGoodsClassifyApi, { manual: true });

    const load = async () => {
        const [error, { data }] = await getPageApi(searchFrom.getFieldsValue());
        if (!error) {
            setDataSource(data.list);
            setTotal(data.total);
        }
    };

    const deleteItem = async (data: GoodsClassifyVO) => {
        const [error, { code, msg }] = await deleteApi(data.id);
        if (error || code !== 0) {
            antdUtils.message?.open({ type: 'error', content: msg ?? '操作失败' });
        } else {
            antdUtils.message?.open({ type: 'success', content: '删除成功' });
        }
        await load();
    };

    const getColumnSearchProps = (placeholder: string): ColumnType<GoodsClassifyVO> => ({
        filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => (
            <div style={{ padding: 8 }} onKeyDown={(e) => e.stopPropagation()}>
                <Input.Search
                    ref={searchInput}
                    placeholder={placeholder}
                    value={selectedKeys[0]}
                    onChange={(e) => {
                        setSelectedKeys(e.target.value && e.target.value !== '' ? [e.target.value] : [])
                    }}
                    onSearch={(value) => {
                        if (value === '' && clearFilters) {
                            clearFilters!!()
                        }
                        confirm();
                    }}
                    onPressEnter={() => confirm()}
                    allowClear
                    style={{ marginBottom: 8, display: 'block' }}
                    enterButton="搜索"
                    size="middle"
                    loading={onSearching}
                />
            </div>
        ),
        filterIcon: (filtered: boolean) => (
            <SearchOutlined style={{ color: filtered ? 'primaryColor' : undefined }} />
        ),
        onFilterDropdownOpenChange: (visible) => {
            if (visible) {
                setTimeout(() => searchInput.current?.select(), 100);
            }
        },
    });

    const defaultColumns: (ColumnTypes[number] & { editable?: boolean; dataIndex: string })[] = [
        {
            title: '类目名称',
            dataIndex: 'classifyName',
            key: 'classifyName',
            align: 'left',
            width: '30%',
            editable: true,
        },
        {
            title: '是否默认属性',
            key: 'isDefault',
            dataIndex: 'isDefault',
            width: 120,
            align: 'center',
            render: (value: number) => {
                return (value === 1 ? <Badge status="success" text="是" /> : <Badge status="error" text="否" />)
            }
        },
        {
            title: '创建时间',
            dataIndex: 'createTime',
            key: 'createTime',
            align: 'center',
            render: (value: number) => {
                return formatDate(new Date(value), "YYYY-mm-dd HH:MM:SS")
            }
        },
        {
            title: t("QkOmYwne" /* 操作 */),
            dataIndex: 'operation',
            key: 'action',
            render: (_, record, index) =>
                dataSource.length >= 1 ? (
                    <Popconfirm title="确认要将该分类删除吗？" onConfirm={() => deleteItem(dataSource[index])}>
                        <a>删除</a>
                    </Popconfirm>
                ) : null,
        },
    ];

    const columns = defaultColumns.map((col) => {
        if (!col.editable) {
            return col;
        }
        return {
            ...col,
            ...getColumnSearchProps("请输入类目名称"),
            onCell: (record: GoodsClassifyVO) => ({
                record,
                editable: col.editable,
                dataIndex: col.dataIndex,
                title: col.title,
                handleSave,

            }),
        };
    });

    const handleSave = async (row: GoodsClassifyVO) => {
        const [error, { code, msg }] = await updateApi(row);
        if (!error && code === 0) {
            const newData = [...dataSource];
            const index = newData.findIndex((item) => row.id === item.id);
            const item = newData[index];
            newData.splice(index, 1, {
                ...item,
                ...row,
            });
            setDataSource(newData);
        } else {
            antdUtils.message?.open({ type: 'error', content: msg ?? '更新失败' })
        }
    };

    const components = {
        body: {
            row: EditableRow,
            cell: EditableCell,
        },
    };

    useEffect(() => {
        load();
    }, []);

    const onReset = () => {
        searchFrom.resetFields()
        load();
    }

    return (
        <>
            <div>
                <div className='flex justify-between content-center mb-2'>
                    <div className='flex justify-normal items-center'>
                        <Form layout='inline' form={searchFrom}>
                            <Form.Item name="classifyName" label="分类名称">
                                <Input className='w-[150px]' placeholder='请输入名称' allowClear />
                            </Form.Item>
                        </Form>
                        <Space.Compact className="ml-5">
                            <Button type='primary' size='large' icon={<SearchOutlined />} onClick={load}> 搜索 </Button>
                            <Button type='primary' size='large' icon={<UndoOutlined />} onClick={onReset}> 重置 </Button>
                        </Space.Compact>
                    </div>
                    <Button type='primary' size='large' icon={<PlusOutlined />} onClick={() => {
                        setVisible(true);
                    }}> 新增分类 </Button>
                </div>
                <Table rowKey="id"
                    scroll={{ x: true }}
                    columns={columns as ColumnTypes}
                    dataSource={dataSource}
                    components={components}
                    rowClassName={() => 'editable-row'}
                    onChange={(pagination) => {

                    }}
                    pagination={{
                        position: ['bottomRight'],
                        current: searchState.pageNo,
                        pageSize: searchState.pageSize,
                        total
                    }} />
            </div>
            <CreateClassify
                visible={visible}
                onCancel={() => {
                    setVisible(false);
                }}
                onSave={(values: GoodsClassifyVO) => {
                    console.log(values)
                    setVisible(false);
                    load();
                }}
            />
        </>
    );
});

