import React, { useState, useEffect, useRef, useContext, forwardRef, useImperativeHandle } from 'react';
import { Space, Table, Button, Input, Modal, Radio, Form, Popconfirm, Badge } from 'antd';
import type { TableColumnsType } from 'antd';
import type { InputRef } from 'antd';
import type { FormInstance } from 'antd/es/form';
import type { ColumnType, TableProps } from 'antd/es/table';
import { t } from '@/utils/i18n';
import { PlusOutlined, ExclamationCircleFilled, SearchOutlined } from '@ant-design/icons';
import type { GoodsAttrVO, GoodsAttrPageReqVO } from '@/models'
import { antdUtils } from '@/utils/antd';
import { useRequest } from '@/hooks/use-request';
import goodsAttrService from '@/request/service/goods-attr';
import { formatDate } from '@/utils/formatTime';
import { useSetState } from 'ahooks';

const AttrEditor = (props: {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (role: GoodsAttrVO) => void;
}) => {

    const { visible, onCancel, onSave } = props;

    const { runAsync: createApi } = useRequest(goodsAttrService.createGoodsAttrApi, { manual: true });


    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();

    useEffect(() => {
        if (visible) {
            form.setFieldValue("attrStatus", 1);
        } else {
            form.resetFields();
        }
    }, [visible]);

    const save = async () => {
        await form.validateFields();
        setSaveLoading(true);
        const fieldValues = form.getFieldsValue();
        const [error, { msg, code }] = await createApi(fieldValues);
        if (!error && code === 0) {
            onSave(fieldValues);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg ?? '操作失败',
            });
        }
        setSaveLoading(false);
    }

    return (
        <>
            <Modal
                open={visible}
                title={"新建"}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                confirmLoading={saveLoading}
                destroyOnClose
            >
                <Form
                    form={form}
                    onFinish={save}
                    labelCol={{ flex: '0 0 120px' }}
                    wrapperCol={{ span: 16 }}
                >
                    <Form.Item name="attrNameCn" label="属性名称(中文):"
                        rules={[
                            {
                                required: true,
                                message: '请输属性中文名称',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>
                    <Form.Item name="attrNameEn" label="属性名称(英文):"
                        rules={[
                            {
                                required: true,
                                message: '请输属性英文名称',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>
                    <Form.Item name="attrStatus" label="可用状态:"
                        rules={[
                            {
                                required: true,
                                message: '请输选择',
                            },
                        ]}
                    >
                        <Radio.Group options={[
                            { value: 1, label: "正常使用" },
                            { value: 2, label: "停止使用" }
                        ]} optionType="default">
                        </Radio.Group>
                    </Form.Item>
                </Form>
            </Modal>
        </>
    )
};

const EditableContext = React.createContext<FormInstance<any> | null>(null);

interface EditableRowProps {
    index: number;
}

const EditableRow: React.FC<EditableRowProps> = ({ index, ...props }) => {
    const [form] = Form.useForm();
    return (
        <Form form={form} component={false}>
            <EditableContext.Provider value={form}>
                <tr {...props} />
            </EditableContext.Provider>
        </Form>
    );
};

interface EditableCellProps {
    title: React.ReactNode;
    editable: boolean;
    children: React.ReactNode;
    dataIndex: keyof GoodsAttrVO;
    record: GoodsAttrVO;
    handleSave: (record: GoodsAttrVO) => void;
}

const EditableCell: React.FC<EditableCellProps> = ({
    title,
    editable,
    children,
    dataIndex,
    record,
    handleSave,
    ...restProps
}) => {
    const [editing, setEditing] = useState(false);
    const inputRef = useRef<InputRef>(null);
    const form = useContext(EditableContext)!;

    useEffect(() => {
        if (editing) {
            inputRef.current!.focus();
        }
    }, [editing]);

    const toggleEdit = () => {
        setEditing(!editing);
        form.setFieldsValue({ [dataIndex]: record[dataIndex] });
    };

    const save = async () => {
        try {
            const values = await form.validateFields();

            toggleEdit();
            handleSave({ ...record, ...values });
        } catch (errInfo) {
            console.log('Save failed:', errInfo);
        }
    };

    let childNode = children;

    if (editable) {
        childNode = editing ? (
            <Form.Item
                style={{ margin: 0 }}
                name={dataIndex}
                rules={[
                    {
                        required: true,
                        message: `${title} is required.`,
                    },
                ]}
            >
                <Input size="middle" ref={inputRef} onPressEnter={save} onBlur={save} />
            </Form.Item>
        ) : (
            <div className="editable-cell-value-wrap" style={{ paddingRight: 24 }} onClick={toggleEdit}>
                {children}
            </div>
        );
    }

    return <td {...restProps}>{childNode}</td>;
};

type EditableTableProps = Parameters<typeof Table>[0];
type ColumnTypes = Exclude<EditableTableProps['columns'], undefined>;

export default forwardRef((props, ref) => {
    useImperativeHandle(ref, () => {
        addItem
    });
    const [visible, setVisible] = useState<boolean>(false);
    const [dataSource, setDataSource] = useState<GoodsAttrVO[]>([]);
    const [searchFrom] = Form.useForm();

    const [searchState, setSearchState] = useSetState<GoodsAttrPageReqVO>({
        pageNo: 1,
        pageSize: 10
    });
    const [total, setTotal] = useState(0)
    const searchInput = useRef<InputRef>(null);
    const [onSearching, setOnSearching] = useState(false);

    const { runAsync: getPageApi } = useRequest(goodsAttrService.getGoodsAttrPageApi, { manual: true });
    const { runAsync: updateApi } = useRequest(goodsAttrService.updateGoodsAttrApi, { manual: true });
    const { runAsync: verifyApi } = useRequest(goodsAttrService.attrNameEnVerifyUnique, { manual: true });
    const { runAsync: deleteApi } = useRequest(goodsAttrService.deleteGoodsAttrApi, { manual: true });

    const load = async () => {
        const [error, { data }] = await getPageApi(searchState);
        if (!error) {
            setDataSource(data.list);
            setTotal(data.total);
        }
    };

    const deleteItem = async (data: GoodsAttrVO) => {
        const [error, { code, msg }] = await deleteApi(data.id);
        if (error || code !== 0) {
            antdUtils.message?.open({ type: 'error', content: msg ?? '操作失败' })
        } else {
            antdUtils.message?.open({ type: 'success', content: '删除成功' })
        }
        await load();
    };

    const addItem = () => {
        console.log("on call add Item")
    }

    const getColumnSearchProps = (placeholder: string): ColumnType<GoodsAttrVO> => ({
        filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => (
            <div style={{ padding: 8 }} onKeyDown={(e) => e.stopPropagation()}>
                <Input.Search
                    ref={searchInput}
                    placeholder={placeholder}
                    value={selectedKeys[0]}
                    onChange={(e) => {
                        setSelectedKeys(e.target.value && e.target.value !== '' ? [e.target.value] : [])
                    }}
                    onSearch={(value) => {
                        if (value === '' && clearFilters) {
                            clearFilters!!()
                        }
                        confirm();
                    }}
                    onPressEnter={() => confirm()}
                    allowClear
                    style={{ marginBottom: 8, display: 'block' }}
                    enterButton="搜索"
                    size="middle"
                    loading={onSearching}
                />
            </div>
        ),
        filterIcon: (filtered: boolean) => (
            <SearchOutlined style={{ color: filtered ? 'primaryColor' : undefined }} />
        ),
        onFilterDropdownOpenChange: (visible) => {
            if (visible) {
                setTimeout(() => searchInput.current?.select(), 100);
            }
        },
    });

    const defaultColumns: (ColumnTypes[number] & { editable?: boolean; dataIndex: string })[] = [
        {
            title: '尺码中文名称',
            dataIndex: 'attrNameCn',
            key: 'attrNameCn',
            align: 'left',
            width: '30%',
            editable: true,
        },
        {
            title: '尺码英文名称',
            dataIndex: 'attrNameEn',
            key: 'attrNameEn',
            align: 'left',
            width: '30%',
            editable: true,
        },
        {
            title: '状态',
            key: 'attrStatus',
            dataIndex: 'attrStatus',
            width: 100,
            align: 'center',
            render: (value: number) => {
                return (value === 1 ? <Badge status="success" text="已开启" /> : <Badge status="error" text="已关闭" />)
            }
        },
        {
            title: '创建时间',
            dataIndex: 'createTime',
            key: 'createTime',
            align: 'center',
            render: (value: number) => {
                return formatDate(new Date(value), "YYYY-mm-dd HH:MM:SS")
            }
        },
        {
            title: t("QkOmYwne" /* 操作 */),
            dataIndex: 'operation',
            key: 'action',
            render: (_, record, index) =>
                dataSource.length >= 1 ? (
                    <Popconfirm title="确认要将该属性删除吗？" onConfirm={() => deleteItem(dataSource[index])}>
                        <a>删除</a>
                    </Popconfirm>
                ) : null,
        },
    ];

    const columns = defaultColumns.map((col) => {
        if (!col.editable) {
            return col;
        }
        return {
            ...col,
            ...getColumnSearchProps("请输入尺码名称"),
            onCell: (record: GoodsAttrVO) => ({
                record,
                editable: col.editable,
                dataIndex: col.dataIndex,
                title: col.title,
                handleSave,
            }),
        };
    });

    const handleSave = async (row: GoodsAttrVO) => {
        const [error, { code, msg }] = await updateApi(row);
        if (!error && code === 0) {
            const newData = [...dataSource];
            const index = newData.findIndex((item) => row.id === item.id);
            const item = newData[index];
            newData.splice(index, 1, {
                ...item,
                ...row,
            });
            setDataSource(newData);
        } else {
            antdUtils.message?.open({ type: 'error', content: msg ?? '更新失败' })
        }
    };

    const components = {
        body: {
            row: EditableRow,
            cell: EditableCell,
        },
    };

    useEffect(() => {
        load();
    }, [searchState]);

    const onReset = () => {
        searchFrom.resetFields()
        load()
    }

    return (
        <>
            <div>
                <div className='flex flex-row-reverse mb-2'>
                    <Button type='primary' size='large' icon={<PlusOutlined />} onClick={() => { setVisible(true) }}> 新增尺码 </Button>
                </div>
                <Table rowKey="id"
                    scroll={{ x: true }}
                    columns={columns as ColumnTypes}
                    dataSource={dataSource}
                    components={components}
                    rowClassName={() => 'editable-row'}
                    onChange={async (pagination, filters) => {
                        const state: GoodsAttrPageReqVO = {
                            attrNameCn: filters.attrNameCn ? filters.attrNameCn[0] as string : undefined,
                            attrNameEn: filters.attrNameEn ? filters.attrNameEn[0] as string : undefined,
                            pageNo: pagination.current ?? 1,
                            pageSize: pagination.pageSize
                        }
                        setSearchState(state);
                    }}
                    pagination={{
                        position: ['bottomRight'],
                        current: searchState.pageNo,
                        pageSize: searchState.pageSize,
                        total
                    }} />
            </div>
            <AttrEditor
                visible={visible}
                onCancel={() => { setVisible(false) }}
                onSave={(data: GoodsAttrVO) => {
                    setVisible(false);
                    load();
                }} />
        </>
    );
});

