import React, { useEffect, useState } from 'react'
import { Form, Input, Modal } from 'antd';
import { useRequest } from '@/hooks/use-request';
import dataSourceService from '@/request/service/data-source';
import { DataSourceConfigVO } from '@/models';
import { antdUtils } from '@/utils/antd';


const layout = {
    labelCol: { span: 4, },
    wrapperCol: { span: 16 }
};

interface DataTypeEditorProps {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (value: DataSourceConfigVO) => void;
    data?: DataSourceConfigVO | null;
}

export default (props: DataTypeEditorProps) => {

    const { visible, onCancel, onSave, data } = props;

    const { runAsync: update } = useRequest(dataSourceService.updateDataSourceConfigApi, { manual: true });
    const { runAsync: create } = useRequest(dataSourceService.createDataSourceConfigApi, { manual: true });

    const isEdit = !!data;


    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();

    useEffect(() => {
        if (visible) {
            if (data) {
                form.setFieldsValue(data);
            }
        } else {
            form.resetFields();
        }
    }, [visible]);

    const save = async () => {
        setSaveLoading(true);
        const fieldValues = form.getFieldsValue();
        const newValue = isEdit ? { ...data, ...fieldValues } : fieldValues;
        const [error, { msg, code }] = isEdit ? await update(newValue) : await create(newValue);
        if (!error && code === 0) {
            onSave(newValue);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg ?? '操作失败',
            });
        }
        setSaveLoading(false);
    }

    return (
        <>
            <Modal
                open={visible}
                title={isEdit ? "编辑" : "新建"}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                confirmLoading={saveLoading}
                destroyOnClose
            >
                <Form
                    form={form}
                    {...layout}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                >
                    <Form.Item name="name" label="数据源名称"
                        rules={[
                            {
                                required: true,
                                message: '请输入数据源名称',
                            },
                        ]}
                    >
                        <Input allowClear/>
                    </Form.Item>

                    <Form.Item name="url" label="数据源连接"
                        rules={[
                            {
                                required: true,
                                message: '请输入数据源连接',
                            },
                        ]}
                    >
                        <Input.TextArea className='h-[150px]' showCount maxLength={200} />
                    </Form.Item>

                    <Form.Item name="username" label="用户名" rules={[
                        {
                            required: true,
                            message: '请输入用户名',
                        },
                    ]}>
                        <Input allowClear/>
                    </Form.Item>

                    <Form.Item name="password" label="密码" rules={[
                        {
                            required: true,
                            message: '请输入密码',
                        },
                    ]}>
                        <Input allowClear />
                    </Form.Item>
                </Form>
            </Modal>
        </>
    )
};
