import React, { useEffect, useState } from 'react'
import { Form, Input, Radio, Modal, InputNumber, ColorPicker } from 'antd';
import { useRequest } from '@/hooks/use-request';
import { antdUtils } from '@/utils/antd';
import service from '@/request/service/error-code';
import type { ErrorCodeVO, ErrorCodePageReqVO } from '@/models';


const layout = {
    labelCol: { span: 4, },
    wrapperCol: { span: 16 }
};

interface DataTypeEditorProps {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (role: ErrorCodeVO) => void;
    data?: ErrorCodeVO | null;
}

export default (props: DataTypeEditorProps) => {

    const { visible, onCancel, onSave, data } = props;
    const { runAsync: update } = useRequest(service.updateErrorCodeApi, { manual: true });
    const { runAsync: create } = useRequest(service.createErrorCodeApi, { manual: true });

    const isEdit = !!data;


    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();

    useEffect(() => {
        if (visible) {
            if (data) {
                form.setFieldsValue(data);
            }
        } else {
            form.resetFields();
        }
    }, [visible]);

    const save = async () => {
        setSaveLoading(true);
        const fieldValues = form.getFieldsValue();
        const cssClass = fieldValues.cssClass ? fieldValues.cssClass.toHex() : '';
        const newValue = isEdit ? { ...data, ...fieldValues, cssClass } : fieldValues;
        const [error, { msg, code }] = isEdit ? await update(newValue) : await create(newValue);
        if (!error && code === 0) {
            onSave(newValue);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg ?? '操作失败',
            });
        }
        setSaveLoading(false);
    }

    return (
        <>
            <Modal
                open={visible}
                title={isEdit ? "编辑" : "新建"}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                confirmLoading={saveLoading}
                destroyOnClose
            >
                <Form
                    form={form}
                    {...layout}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                >

                    <Form.Item name="type" label="错误码类型">
                        <Radio.Group options={[
                            { value: 1, label: "自动生成" },
                            { value: 2, label: "手动编辑" }
                        ]} optionType="button">
                        </Radio.Group>
                    </Form.Item>

                    <Form.Item name="applicationName" label="应用名"
                        rules={[
                            {
                                required: true,
                                message: '请输入应用名',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="code" label="错误码编码"
                        rules={[
                            {
                                required: true,
                                message: '请输入错误码编码',
                            },
                        ]}>
                        <InputNumber min={0} />
                    </Form.Item>
                    <Form.Item name="message" label="错误码提示"
                        rules={[
                            {
                                required: true,
                                message: '请输入错误码编码',
                            },
                        ]}>
                        <Input.TextArea showCount maxLength={100} />
                    </Form.Item>
                </Form>
            </Modal>
        </>
    )
};
