import React, { useEffect, useState } from 'react'
import { Form, Input, InputNumber, Radio, Modal, Select, TreeSelect } from 'antd';
import { EyeTwoTone, EyeInvisibleOutlined } from '@ant-design/icons';
import positonService from '@/request/service/positon';
import departmentService from '@/request/service/deparatment';
import userService from '@/request/service/user';
import { useRequest } from '@/hooks/use-request';
import type { DepartmentVO, UserVO } from '@/models'
import { antdUtils } from '@/utils/antd';
import type { DefaultOptionType } from 'antd/es/select';

const layout = {
    labelCol: { span: 4, },
    wrapperCol: { span: 16 },
};

interface EditorProps {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (role: UserVO) => void;
    data?: UserVO | null;
}

const toDepartmentTree = (data: DepartmentVO[]) => {
    const root: DepartmentVO[] = data.filter(item => {
        return !data.some(it => it.id === item.parentId)
    });
    const toDepartmentTreeItem = (item: DepartmentVO): DefaultOptionType => {
        return {
            label: item.name,
            value: item.id,
            children: data.filter(it => it.parentId === item.id).map(it => {
                return toDepartmentTreeItem(it)
            })
        }
    }
    return root.map(item => {
        return toDepartmentTreeItem(item)
    })
}


const UserEditor: React.FC<EditorProps> = (props) => {

    const { visible, onCancel, onSave, data } = props;

    const { runAsync: updateUserApi } = useRequest(userService.updateUserApi, { manual: true });
    const { runAsync: createUserApi } = useRequest(userService.createUserApi, { manual: true });
    const { data: simpleDepartment, run: listSimpleDeptApi } = useRequest(departmentService.listSimpleDeptApi, { manual: true });
    const { data: simplePositions, run: listSimplePostsApi } = useRequest(positonService.listSimplePostsApi, { manual: true });

    const isEdit = !!data;

    const [shouldUpdateEditPassword, setUpdateEditPassword] = useState(false);
    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();

    useEffect(() => {
        if (visible) {
            if (data) {
                form.setFieldsValue(data);
            }
            setUpdateEditPassword(true)
        } else {
            form.resetFields();
        }
    }, [visible]);

    const save = async () => {
        setSaveLoading(true);
        const fieldValues = form.getFieldsValue();
        const newValue = isEdit ? { ...data, ...fieldValues } : fieldValues;
        const [error, { msg, code }] = isEdit ? await updateUserApi(newValue) : await createUserApi(newValue);
        if (!error && code === 0) {
            onSave(newValue);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg ?? '操作失败',
            });
        }
        setSaveLoading(false);
    }

    useEffect(() => {
        listSimpleDeptApi();
        listSimplePostsApi();
    }, [listSimplePostsApi, listSimpleDeptApi]);

    return (
        <>
            <Modal
                open={visible}
                title= {<div className='h-50px'>{isEdit? "编辑用户": "添加用户"}</div>}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                destroyOnClose
                confirmLoading= {saveLoading}
            >
                <Form
                    form={form}
                    {...layout}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                >
                    <Form.Item name="username" label="用户账号"
                        hidden={isEdit}
                        rules={[
                            {
                                required: true,
                                message: '请输入用户账号',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item noStyle
                        shouldUpdate={() => {
                            const shouldUpdate = shouldUpdateEditPassword;
                            if(shouldUpdateEditPassword) {
                                setUpdateEditPassword(false)
                            }
                            return shouldUpdate
                        }}
                    >
                        {() => {
                            return !isEdit ? (
                                <Form.Item name="password" label="用户密码"
                                    rules={[
                                        {
                                            required: true,
                                            message: '请输入用户密码',
                                        },
                                    ]}
                                >
                                    <Input.Password iconRender={(visible) => (visible ? <EyeTwoTone /> : <EyeInvisibleOutlined />)} />
                                </Form.Item>
                            ) : null
                        }}
                    </Form.Item>

                    <Form.Item name="nickname" label="用户昵称"
                        rules={[
                            {
                                required: true,
                                message: '请输入用户昵称',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>
                    <Form.Item name="email" label="邮箱"
                        rules={[
                            {
                                required: true,
                                message: '请输入邮箱',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="mobile" label="联系电话"
                        rules={[
                            {
                                required: true,
                                message: '请输入电话号码',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item
                        name="deptId"
                        label="部门"
                    >
                        <TreeSelect treeData={toDepartmentTree(simpleDepartment?.data ?? [])} />
                    </Form.Item>

                    <Form.Item name="postIds" label="岗位" >
                        <Select mode="multiple" placeholder="请选择用户岗位" options={simplePositions?.data.map(item => {
                            return {
                                label: item.name,
                                value: item.id,
                            }
                        })}>
                        </Select>
                    </Form.Item>

                    <Form.Item name="remark" label="备注">
                        <Input />
                    </Form.Item>

                    <Form.Item name="status" label="状态">
                        <Radio.Group options={[
                            { value: 0, label: "开启" },
                            { value: 1, label: "关闭" }
                        ]} optionType="default">
                        </Radio.Group>
                    </Form.Item>
                </Form>
            </Modal>
        </>
    )
}

export default UserEditor;
